@@ -19,7 +19,7 @@ from .mesonlib import MesonException, commonpath
 from .mesonlib import default_libdir, default_libexecdir, default_prefix
 import ast
 
-version = '0.40.0'
+version = '0.41.0'
 backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'xcode']
 
 class UserOption:
@@ -160,7 +160,7 @@ class CoreData:
         self.wrap_mode = options.wrap_mode
         self.compilers = OrderedDict()
         self.cross_compilers = OrderedDict()
-        self.deps = {}
+        self.deps = OrderedDict()
         self.modules = {}
         # Only to print a warning if it changes between Meson invocations.
         self.pkgconf_envvar = os.environ.get('PKG_CONFIG_PATH', '')
@@ -334,6 +334,7 @@ forbidden_target_names = {'clean': None,
                           'clean-ctlist': None,
                           'clean-gcno': None,
                           'clean-gcda': None,
+                          'coverage': None,
                           'coverage-text': None,
                           'coverage-xml': None,
                           'coverage-html': None,
@@ -347,4 +348,6 @@ forbidden_target_names = {'clean': None,
                           'build.ninja': None,
                           'scan-build': None,
                           'reconfigure': None,
+                          'dist': None,
+                          'distcheck': None,
                           }
