@@ -21,7 +21,7 @@ import sys
 import copy
 import subprocess
 from . import ModuleReturnValue
-from ..mesonlib import MesonException, Popen_safe
+from ..mesonlib import MesonException, OrderedSet, Popen_safe
 from ..dependencies import Dependency, PkgConfigDependency, InternalDependency
 from .. import mlog
 from .. import mesonlib
@@ -37,10 +37,11 @@ from . import ExtensionModule
 #
 # https://github.com/ninja-build/ninja/issues/1184
 # https://bugzilla.gnome.org/show_bug.cgi?id=774368
-gresource_dep_needed_version = '>= 2.52.0'
+gresource_dep_needed_version = '>= 2.51.1'
 
 native_glib_version = None
 girwarning_printed = False
+gdbuswarning_printed = False
 gresource_warning_printed = False
 _gir_has_extra_lib_arg = None
 
@@ -67,21 +68,31 @@ class GnomeModule(ExtensionModule):
         if native_glib_version is None:
             glib_dep = PkgConfigDependency('glib-2.0', state.environment,
                                            {'native': True})
-            native_glib_version = glib_dep.get_modversion()
+            native_glib_version = glib_dep.get_version()
         return native_glib_version
 
     def __print_gresources_warning(self, state):
         global gresource_warning_printed
         if not gresource_warning_printed:
             if not mesonlib.version_compare(self._get_native_glib_version(state), gresource_dep_needed_version):
-                mlog.warning('''GLib compiled dependencies do not work reliably with
-the current version of GLib. See the following upstream issue:''',
+                mlog.warning('GLib compiled dependencies do not work reliably with \n'
+                             'the current version of GLib. See the following upstream issue:',
                              mlog.bold('https://bugzilla.gnome.org/show_bug.cgi?id=774368'))
             gresource_warning_printed = True
         return []
 
+    @staticmethod
+    def _print_gdbus_warning():
+        global gdbuswarning_printed
+        if not gdbuswarning_printed:
+            mlog.warning('Code generated with gdbus_codegen() requires the root directory be added to\n'
+                         '  include_directories of targets with GLib < 2.51.3:',
+                         mlog.bold('https://github.com/mesonbuild/meson/issues/1387'))
+            gdbuswarning_printed = True
+
     def compile_resources(self, state, args, kwargs):
         self.__print_gresources_warning(state)
+        glib_version = self._get_native_glib_version(state)
 
         cmd = ['glib-compile-resources', '@INPUT@']
 
@@ -90,26 +101,48 @@ the current version of GLib. See the following upstream issue:''',
             source_dirs = [source_dirs]
 
         if len(args) < 2:
-            raise MesonException('Not enough arguments; The name of the resource and the path to the XML file are required')
+            raise MesonException('Not enough arguments; the name of the resource '
+                                 'and the path to the XML file are required')
 
         dependencies = kwargs.pop('dependencies', [])
         if not isinstance(dependencies, list):
             dependencies = [dependencies]
-
-        glib_version = self._get_native_glib_version(state)
-        if not mesonlib.version_compare(glib_version, gresource_dep_needed_version):
-            if len(dependencies) > 0:
-                raise MesonException('''The "dependencies" argument of gnome.compile_resources()
-can not be used with the current version of glib-compiled-resources, due to
-<https://bugzilla.gnome.org/show_bug.cgi?id=774368>''')
+        # Validate dependencies
+        for (ii, dep) in enumerate(dependencies):
+            if hasattr(dep, 'held_object'):
+                dependencies[ii] = dep = dep.held_object
+            if not isinstance(dep, (mesonlib.File, build.CustomTarget)):
+                m = 'Unexpected dependency type {!r} for gnome.compile_resources() ' \
+                    '"dependencies" argument.\nPlease pass the return value of ' \
+                    'custom_target() or configure_file()'
+                raise MesonException(m.format(dep))
+            if isinstance(dep, build.CustomTarget):
+                if not mesonlib.version_compare(glib_version, gresource_dep_needed_version):
+                    m = 'The "dependencies" argument of gnome.compile_resources() can not\n' \
+                        'be used with the current version of glib-compile-resources due to\n' \
+                        '<https://bugzilla.gnome.org/show_bug.cgi?id=774368>'
+                    raise MesonException(m)
 
         ifile = args[1]
         if isinstance(ifile, mesonlib.File):
-            ifile = os.path.join(ifile.subdir, ifile.fname)
+            # glib-compile-resources will be run inside the source dir,
+            # so we need either 'src_to_build' or the absolute path.
+            # Absolute path is the easiest choice.
+            if ifile.is_built:
+                ifile = os.path.join(state.environment.get_build_dir(), ifile.subdir, ifile.fname)
+            else:
+                ifile = os.path.join(ifile.subdir, ifile.fname)
         elif isinstance(ifile, str):
             ifile = os.path.join(state.subdir, ifile)
+        elif isinstance(ifile, (interpreter.CustomTargetHolder,
+                                interpreter.GeneratedObjectsHolder)):
+            m = 'Resource xml files generated at build-time cannot be used ' \
+                'with gnome.compile_resources() because we need to scan ' \
+                'the xml for dependencies. Use configure_file() instead ' \
+                'to generate it at configure-time.'
+            raise MesonException(m)
         else:
-            raise RuntimeError('Unreachable code.')
+            raise MesonException('Invalid file argument: {!r}'.format(ifile))
 
         depend_files, depends, subdirs = self._get_gresource_dependencies(
             state, ifile, source_dirs, dependencies)
@@ -121,7 +154,7 @@ can not be used with the current version of glib-compiled-resources, due to
         # Ensure build directories of generated deps are included
         source_dirs += subdirs
 
-        for source_dir in set(source_dirs):
+        for source_dir in OrderedSet(source_dirs):
             cmd += ['--sourcedir', source_dir]
 
         if 'c_name' in kwargs:
@@ -183,13 +216,6 @@ can not be used with the current version of glib-compiled-resources, due to
         return ModuleReturnValue(rv, rv)
 
     def _get_gresource_dependencies(self, state, input_file, source_dirs, dependencies):
-        for dep in dependencies:
-            if not isinstance(dep, interpreter.CustomTargetHolder) and not \
-                    isinstance(dep, mesonlib.File):
-                raise MesonException(
-                    'Unexpected dependency type for gnome.compile_resources() '
-                    '"dependencies" argument. Please pass the output of '
-                    'custom_target() or configure_file().')
 
         cmd = ['glib-compile-resources',
                input_file,
@@ -199,9 +225,10 @@ can not be used with the current version of glib-compiled-resources, due to
             cmd += ['--sourcedir', os.path.join(state.subdir, source_dir)]
         cmd += ['--sourcedir', state.subdir] # Current dir
 
-        pc, stdout = Popen_safe(cmd, cwd=state.environment.get_source_dir())[0:2]
+        pc, stdout, stderr = Popen_safe(cmd, cwd=state.environment.get_source_dir())
         if pc.returncode != 0:
-            mlog.warning('glib-compile-resources has failed to get the dependencies for {}'.format(cmd[1]))
+            m = 'glib-compile-resources failed to get dependencies for {}:\n{}'
+            mlog.warning(m.format(cmd[1], stderr))
             raise subprocess.CalledProcessError(pc.returncode, cmd)
 
         dep_files = stdout.split('\n')[:-1]
@@ -226,6 +253,8 @@ can not be used with the current version of glib-compiled-resources, due to
             missing_basename = os.path.basename(missing)
 
             for dep in dependencies:
+                if hasattr(dep, 'held_object'):
+                    dep = dep.held_object
                 if isinstance(dep, mesonlib.File):
                     if dep.fname == missing_basename:
                         found = True
@@ -233,18 +262,20 @@ can not be used with the current version of glib-compiled-resources, due to
                         dep_files.append(dep)
                         subdirs.append(dep.subdir)
                         break
-                elif isinstance(dep, interpreter.CustomTargetHolder):
-                    if dep.held_object.get_basename() == missing_basename:
+                elif isinstance(dep, build.CustomTarget):
+                    if dep.get_basename() == missing_basename:
                         found = True
                         dep_files.remove(missing)
                         dep_files.append(
                             mesonlib.File(
                                 is_built=True,
-                                subdir=dep.held_object.get_subdir(),
-                                fname=dep.held_object.get_basename()))
-                        depends.append(dep.held_object)
-                        subdirs.append(dep.held_object.get_subdir())
+                                subdir=dep.get_subdir(),
+                                fname=dep.get_basename()))
+                        depends.append(dep)
+                        subdirs.append(dep.get_subdir())
                         break
+                else:
+                    raise RuntimeError('Unreachable code.')
 
             if not found:
                 raise MesonException(
@@ -255,24 +286,34 @@ can not be used with the current version of glib-compiled-resources, due to
 
         return dep_files, depends, subdirs
 
-    def _get_link_args(self, state, lib, depends=None, include_rpath=False):
-        if gir_has_extra_lib_arg():
-            link_command = ['--extra-library=%s' % lib.name]
+    def _get_link_args(self, state, lib, depends=None, include_rpath=False,
+                       use_gir_args=False):
+        # Construct link args
+        if gir_has_extra_lib_arg() and use_gir_args:
+            link_command = ['--extra-library=' + lib.name]
         else:
-            link_command = ['-l%s' % lib.name]
+            link_command = ['-l' + lib.name]
         if isinstance(lib, build.SharedLibrary):
-            libdir = os.path.join(state.environment.get_build_dir(), lib.subdir)
-            link_command += ['-L%s' % libdir]
+            libdir = state.backend.get_target_dir(lib)
+            link_command.append('-L' + libdir)
+            # Needed for the following binutils bug:
+            # https://github.com/mesonbuild/meson/issues/1911
+            # However, g-ir-scanner does not understand -Wl,-rpath
+            # so we need to use -L instead
+            for d in state.backend.determine_rpath_dirs(lib):
+                d = os.path.join(state.environment.get_build_dir(), d)
+                link_command.append('-L' + d)
             if include_rpath:
-                link_command += ['-Wl,-rpath %s' % libdir]
+                link_command.append('-Wl,-rpath,' + libdir)
             if depends:
                 depends.append(lib)
         return link_command
 
-    def _get_dependencies_flags(self, deps, state, depends=None, include_rpath=False):
-        cflags = set()
-        ldflags = set()
-        gi_includes = set()
+    def _get_dependencies_flags(self, deps, state, depends=None, include_rpath=False,
+                                use_gir_args=False):
+        cflags = OrderedSet()
+        ldflags = OrderedSet()
+        gi_includes = OrderedSet()
         if not isinstance(deps, list):
             deps = [deps]
 
@@ -280,14 +321,16 @@ can not be used with the current version of glib-compiled-resources, due to
             if hasattr(dep, 'held_object'):
                 dep = dep.held_object
             if isinstance(dep, InternalDependency):
-                cflags.update(get_include_args(state.environment, dep.include_directories))
+                cflags.update(get_include_args(dep.include_directories))
                 for lib in dep.libraries:
                     ldflags.update(self._get_link_args(state, lib.held_object, depends, include_rpath))
-                    libdepflags = self._get_dependencies_flags(lib.held_object.get_external_deps(), state, depends, include_rpath)
+                    libdepflags = self._get_dependencies_flags(lib.held_object.get_external_deps(), state, depends, include_rpath,
+                                                               use_gir_args)
                     cflags.update(libdepflags[0])
                     ldflags.update(libdepflags[1])
                     gi_includes.update(libdepflags[2])
-                extdepflags = self._get_dependencies_flags(dep.ext_deps, state, depends, include_rpath)
+                extdepflags = self._get_dependencies_flags(dep.ext_deps, state, depends, include_rpath,
+                                                           use_gir_args)
                 cflags.update(extdepflags[0])
                 ldflags.update(extdepflags[1])
                 gi_includes.update(extdepflags[2])
@@ -314,7 +357,7 @@ can not be used with the current version of glib-compiled-resources, due to
                     # Hack to avoid passing some compiler options in
                     if lib.startswith("-W"):
                         continue
-                    if gir_has_extra_lib_arg():
+                    if gir_has_extra_lib_arg() and use_gir_args:
                         lib = lib.replace('-l', '--extra-library=')
                     ldflags.update([lib])
 
@@ -324,7 +367,8 @@ can not be used with the current version of glib-compiled-resources, due to
                         gi_includes.update([girdir])
             elif isinstance(dep, (build.StaticLibrary, build.SharedLibrary)):
                 for incd in dep.get_include_dirs():
-                    cflags.update(incd.get_incdirs())
+                    for idir in incd.get_incdirs():
+                        cflags.update(["-I%s" % idir])
             else:
                 mlog.log('dependency %s not handled to build gir files' % dep)
                 continue
@@ -350,11 +394,7 @@ can not be used with the current version of glib-compiled-resources, due to
                                                    {'native': True})
             pkgargs = self.gir_dep.get_compile_args()
         except Exception:
-            global girwarning_printed
-            if not girwarning_printed:
-                mlog.warning('gobject-introspection dependency was not found, disabling gir generation.')
-                girwarning_printed = True
-            return ModuleReturnValue(None, [])
+            raise MesonException('gobject-introspection dependency was not found, gir cannot be generated.')
         ns = kwargs.pop('namespace')
         nsversion = kwargs.pop('nsversion')
         libsources = kwargs.pop('sources')
@@ -362,7 +402,7 @@ can not be used with the current version of glib-compiled-resources, due to
         depends = [girtarget]
         gir_inc_dirs = []
 
-        scan_command = giscanner.get_command() + ['@INPUT@']
+        scan_command = [giscanner, '@INPUT@']
         scan_command += pkgargs
         scan_command += ['--no-libtool', '--namespace=' + ns, '--nsversion=' + nsversion, '--warn-all',
                          '--output', '@OUTPUT@']
@@ -371,14 +411,15 @@ can not be used with the current version of glib-compiled-resources, due to
         scan_command += extra_args
         scan_command += ['-I' + os.path.join(state.environment.get_source_dir(), state.subdir),
                          '-I' + os.path.join(state.environment.get_build_dir(), state.subdir)]
-        scan_command += get_include_args(state.environment, girtarget.get_include_dirs())
+        scan_command += get_include_args(girtarget.get_include_dirs())
 
         if 'link_with' in kwargs:
             link_with = kwargs.pop('link_with')
             if not isinstance(link_with, list):
                 link_with = [link_with]
             for link in link_with:
-                scan_command += self._get_link_args(state, link.held_object, depends)
+                scan_command += self._get_link_args(state, link.held_object, depends,
+                                                    use_gir_args=True)
 
         if 'includes' in kwargs:
             includes = kwargs.pop('includes')
@@ -403,19 +444,21 @@ can not be used with the current version of glib-compiled-resources, due to
                         'Gir includes must be str, GirTarget, or list of them')
 
         cflags = []
-        if state.global_args.get('c'):
-            cflags += state.global_args['c']
-        if state.project_args.get('c'):
-            cflags += state.project_args['c']
-        if 'c' in state.compilers:
-            compiler = state.compilers['c']
+        for lang, compiler in girtarget.compilers.items():
+            # XXX: Can you use g-i with any other language?
+            if lang in ('c', 'cpp', 'objc', 'objcpp', 'd'):
+                break
+        else:
+            lang = None
+            compiler = None
+        if lang and compiler:
+            if state.global_args.get(lang):
+                cflags += state.global_args[lang]
+            if state.project_args.get(lang):
+                cflags += state.project_args[lang]
             sanitize = compiler.get_options().get('b_sanitize')
             if sanitize:
                 cflags += compilers.sanitizer_compile_args(sanitize)
-        if cflags:
-            scan_command += ['--cflags-begin']
-            scan_command += cflags
-            scan_command += ['--cflags-end']
         if kwargs.get('symbol_prefix'):
             sym_prefix = kwargs.pop('symbol_prefix')
             if not isinstance(sym_prefix, str):
@@ -478,8 +521,12 @@ can not be used with the current version of glib-compiled-resources, due to
         # ldflags will be misinterpreted by gir scanner (showing
         # spurious dependencies) but building GStreamer fails if they
         # are not used here.
-        cflags, ldflags, gi_includes = self._get_dependencies_flags(deps, state, depends)
-        scan_command += list(cflags)
+        dep_cflags, ldflags, gi_includes = self._get_dependencies_flags(deps, state, depends,
+                                                                        use_gir_args=True)
+        cflags += list(dep_cflags)
+        scan_command += ['--cflags-begin']
+        scan_command += cflags
+        scan_command += ['--cflags-end']
         # need to put our output directory first as we need to use the
         # generated libraries instead of any possibly installed system/prefix
         # ones.
@@ -496,14 +543,20 @@ can not be used with the current version of glib-compiled-resources, due to
             if not isinstance(incd.held_object, (str, build.IncludeDirs)):
                 raise MesonException(
                     'Gir include dirs should be include_directories().')
-        scan_command += get_include_args(state.environment, inc_dirs)
-        scan_command += get_include_args(state.environment, gir_inc_dirs + inc_dirs,
-                                         prefix='--add-include-path=')
+        scan_command += get_include_args(inc_dirs)
+        scan_command += get_include_args(gir_inc_dirs + inc_dirs, prefix='--add-include-path=')
 
         if isinstance(girtarget, build.Executable):
             scan_command += ['--program', girtarget]
         elif isinstance(girtarget, build.SharedLibrary):
             libname = girtarget.get_basename()
+            # Needed for the following binutils bug:
+            # https://github.com/mesonbuild/meson/issues/1911
+            # However, g-ir-scanner does not understand -Wl,-rpath
+            # so we need to use -L instead
+            for d in state.backend.determine_rpath_dirs(girtarget):
+                d = os.path.join(state.environment.get_build_dir(), d)
+                scan_command.append('-L' + d)
             scan_command += ['--library', libname]
         scankwargs = {'output': girfile,
                       'input': libsources,
@@ -516,9 +569,8 @@ can not be used with the current version of glib-compiled-resources, due to
         scan_target = GirTarget(girfile, state.subdir, scankwargs)
 
         typelib_output = '%s-%s.typelib' % (ns, nsversion)
-        typelib_cmd = gicompiler.get_command() + [scan_target, '--output', '@OUTPUT@']
-        typelib_cmd += get_include_args(state.environment, gir_inc_dirs,
-                                        prefix='--includedir=')
+        typelib_cmd = [gicompiler, scan_target, '--output', '@OUTPUT@']
+        typelib_cmd += get_include_args(gir_inc_dirs, prefix='--includedir=')
         for incdir in typelib_includes:
             typelib_cmd += ["--includedir=" + incdir]
 
@@ -535,12 +587,12 @@ can not be used with the current version of glib-compiled-resources, due to
         return ModuleReturnValue(rv, rv)
 
     def compile_schemas(self, state, args, kwargs):
-        if len(args) != 0:
+        if args:
             raise MesonException('Compile_schemas does not take positional arguments.')
         srcdir = os.path.join(state.build_to_src, state.subdir)
         outdir = state.subdir
 
-        cmd = find_program('glib-compile-schemas', 'gsettings-compile').get_command()
+        cmd = [find_program('glib-compile-schemas', 'gsettings-compile')]
         cmd += ['--targetdir', outdir, srcdir]
         kwargs['command'] = cmd
         kwargs['input'] = []
@@ -666,6 +718,7 @@ can not be used with the current version of glib-compiled-resources, due to
         args += self._unpack_args('--scanobjsargs=', 'scanobjs_args', kwargs)
         args += self._unpack_args('--gobjects-types-file=', 'gobject_typesfile', kwargs, state)
         args += self._unpack_args('--fixxrefargs=', 'fixxref_args', kwargs)
+        args += self._unpack_args('--mkdbargs=', 'mkdb_args', kwargs)
         args += self._unpack_args('--html-assets=', 'html_assets', kwargs, state)
         args += self._unpack_args('--content-files=', 'content_files', kwargs, state)
         args += self._unpack_args('--expand-content-files=', 'expand_content_files', kwargs, state)
@@ -687,7 +740,7 @@ can not be used with the current version of glib-compiled-resources, due to
             if not isinstance(incd.held_object, (str, build.IncludeDirs)):
                 raise MesonException(
                     'Gir include dirs should be include_directories().')
-        cflags.update(get_include_args(state.environment, inc_dirs))
+        cflags.update(get_include_args(inc_dirs))
         if cflags:
             args += ['--cflags=%s' % ' '.join(cflags)]
         if ldflags:
@@ -699,7 +752,7 @@ can not be used with the current version of glib-compiled-resources, due to
 
         return args
 
-    def gtkdoc_html_dir(self, state, args, kwarga):
+    def gtkdoc_html_dir(self, state, args, kwargs):
         if len(args) != 1:
             raise MesonException('Must have exactly one argument.')
         modulename = args[0]
@@ -734,12 +787,20 @@ can not be used with the current version of glib-compiled-resources, due to
         namebase = args[0]
         xml_file = args[1]
         target_name = namebase + '-gdbus'
-        cmd = find_program('gdbus-codegen', target_name).get_command()
+        cmd = [find_program('gdbus-codegen', target_name)]
         if 'interface_prefix' in kwargs:
             cmd += ['--interface-prefix', kwargs.pop('interface_prefix')]
         if 'namespace' in kwargs:
             cmd += ['--c-namespace', kwargs.pop('namespace')]
-        cmd += ['--generate-c-code', '@OUTDIR@/' + namebase, '@INPUT@']
+        if kwargs.get('object_manager', False):
+            cmd += ['--c-generate-object-manager']
+
+        # https://git.gnome.org/browse/glib/commit/?id=ee09bb704fe9ccb24d92dd86696a0e6bb8f0dc1a
+        if mesonlib.version_compare(self._get_native_glib_version(state), '>= 2.51.3'):
+            cmd += ['--output-directory', '@OUTDIR@', '--generate-c-code', namebase, '@INPUT@']
+        else:
+            self._print_gdbus_warning()
+            cmd += ['--generate-c-code', '@OUTDIR@/' + namebase, '@INPUT@']
         outputs = [namebase + '.c', namebase + '.h']
         custom_kwargs = {'input': xml_file,
                          'output': outputs,
@@ -792,7 +853,7 @@ can not be used with the current version of glib-compiled-resources, due to
             elif arg not in known_custom_target_kwargs:
                 raise MesonException(
                     'Mkenums does not take a %s keyword argument.' % (arg, ))
-        cmd = find_program('glib-mkenums', 'mkenums').get_command() + cmd
+        cmd = [find_program('glib-mkenums', 'mkenums')] + cmd
         custom_kwargs = {}
         for arg in known_custom_target_kwargs:
             if arg in kwargs:
@@ -857,6 +918,7 @@ can not be used with the current version of glib-compiled-resources, due to
         }
         custom_kwargs.update(kwargs)
         return build.CustomTarget(output, state.subdir, custom_kwargs,
+                                  # https://github.com/mesonbuild/meson/issues/973
                                   absolute_paths=True)
 
     def genmarshal(self, state, args, kwargs):
@@ -874,7 +936,7 @@ can not be used with the current version of glib-compiled-resources, due to
             raise MesonException(
                 'Sources keyword argument must be a string or array.')
 
-        cmd = find_program('glib-genmarshal', output + '_genmarshal').get_command()
+        cmd = [find_program('glib-genmarshal', output + '_genmarshal')]
         known_kwargs = ['internal', 'nostdinc', 'skip_source', 'stdinc',
                         'valist_marshallers']
         known_custom_target_kwargs = ['build_always', 'depends',
@@ -895,13 +957,19 @@ can not be used with the current version of glib-compiled-resources, due to
 
         custom_kwargs = {
             'input': sources,
-            'capture': True,
         }
+
+        # https://github.com/GNOME/glib/commit/0fbc98097fac4d3e647684f344e508abae109fdf
+        if mesonlib.version_compare(self._get_native_glib_version(state), '>= 2.51.0'):
+            cmd += ['--output', '@OUTPUT@']
+        else:
+            custom_kwargs['capture'] = True
+
         for arg in known_custom_target_kwargs:
             if arg in kwargs:
                 custom_kwargs[arg] = kwargs[arg]
 
-        custom_kwargs['command'] = cmd + ['--header', '--body', '@INPUT@']
+        custom_kwargs['command'] = cmd + ['--body', '@INPUT@']
         custom_kwargs['output'] = output + '.c'
         body = build.CustomTarget(output + '_c', state.subdir, custom_kwargs)
 
@@ -959,7 +1027,7 @@ can not be used with the current version of glib-compiled-resources, due to
                                           target.get_subdir())
                     outdir = os.path.join(state.environment.get_build_dir(),
                                           target.get_subdir())
-                    outfile = target.output[0][:-5] # Strip .vapi
+                    outfile = target.get_outputs()[0][:-5] # Strip .vapi
                     ret.append('--vapidir=' + outdir)
                     ret.append('--girdir=' + outdir)
                     ret.append('--pkg=' + outfile)
@@ -1003,7 +1071,7 @@ can not be used with the current version of glib-compiled-resources, due to
         build_dir = os.path.join(state.environment.get_build_dir(), state.subdir)
         source_dir = os.path.join(state.environment.get_source_dir(), state.subdir)
         pkg_cmd, vapi_depends, vapi_packages, vapi_includes = self._extract_vapi_packages(state, kwargs)
-        cmd = find_program('vapigen', 'Vaapi').get_command()
+        cmd = [find_program('vapigen', 'Vaapi')]
         cmd += ['--quiet', '--library=' + library, '--directory=' + build_dir]
         cmd += self._vapi_args_to_command('--vapidir=', 'vapi_dirs', kwargs)
         cmd += self._vapi_args_to_command('--metadatadir=', 'metadata_dirs', kwargs)
@@ -1026,7 +1094,7 @@ can not be used with the current version of glib-compiled-resources, due to
                 link_with += self._get_vapi_link_with(i.held_object)
                 subdir = os.path.join(state.environment.get_build_dir(),
                                       i.held_object.get_subdir())
-                gir_file = os.path.join(subdir, i.held_object.output[0])
+                gir_file = os.path.join(subdir, i.held_object.get_outputs()[0])
                 cmd.append(gir_file)
             else:
                 raise MesonException('Input must be a str or GirTarget')
